const unified = require('unified')
const remarkParse = require('remark-parse')
const remarkRehype = require('remark-rehype')
const rehypeStringify = require('rehype-stringify')
const markdownDefaults = require('../index')

/**
 * Transforms a string of markdown into a string of HTML
 *
 * @param {string} markdown - A string of code
 * @param {object} options - Options to configure plugins
 * @param {array} options.remarkPlugins - Use custom remark plugins. Overrides defaultPlugins.
 * @param {array} options.rehypePlugins - Use custom rehype plugins. Overrides defaultPlugins.
 * @param {*} options.defaultPlugins - Pass true to our default plugin set from @hashicorp/nextjs-scripts/markdown. Pass an object to use our default plugin set. If an object is passed, it supports all of the same options as `@hashicorp/nextjs-scripts/markdown`
 *
 */
async function markdownToHtml(markdown, options = {}) {
  //  Parse the consumer's desired remark and rehype plugins
  const { defaultPlugins } = options
  const defaultsOptions = defaultPlugins === true ? {} : defaultPlugins
  const defaults = markdownDefaults(defaultsOptions)
  const remarkPlugins =
    options.remarkPlugins || (defaultPlugins ? defaults.remarkPlugins : [])
  const rehypePlugins =
    options.rehypePlugins || (defaultPlugins ? defaults.rehypePlugins : [])
  //  Configure a unified processor
  var processor = unified()
  //  Parse markdown to an [`mdast`](https://github.com/syntax-tree/mdast) syntax tree
  processor.use(remarkParse)
  //  Use the specified "remark plugins"
  //  * Note that "remark plugins" are just unified "attachers" that
  //  * transform [`mdast`](https://github.com/syntax-tree/mdast) syntax trees
  remarkPlugins.forEach((entry) => {
    const [plugin, options] = Array.isArray(entry) ? entry : [entry, undefined]
    processor.use(plugin, options)
  })
  //  Parse the [`mdast`](https://github.com/syntax-tree/mdast) syntax tree
  //  into a [`hast`](https://github.com/syntax-tree/hast) syntax tree
  processor.use(remarkRehype, { allowDangerousHtml: true })
  //  Use the specified "rehype plugins"
  //  * Note that "rehype plugins" are just unified "attachers" that
  //  * transform [`hast`](https://github.com/syntax-tree/hast) syntax trees
  rehypePlugins.forEach((entry) => {
    const [plugin, options] = Array.isArray(entry) ? entry : [entry, undefined]
    processor.use(plugin, options)
  })
  //  Transform the [`hast`](https://github.com/syntax-tree/hast) syntax tree
  //  into a string of HTML with rehype-stringify
  processor.use(rehypeStringify, { allowDangerousHtml: true })
  //  Use our configured processor to get from markdown to HTML
  const htmlOutput = String(await processor.process(markdown))
  return htmlOutput
}

module.exports = markdownToHtml
